#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h,
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e6|e7|e9|e10|e14>:<zlib|nofcpp|mpich|mvapich2]> [debug|prof] [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir()
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

set_qualifiers()
{
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi
    qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
    (( seen_bqual = 0 ))
    for qual in "${qlist[@]}"; do
        case $qual in
            e[1-9]*)
                cqual=$qual
                ;;
            mpich)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, nofcpp and zlib are mutually exclusive"
                bqual=$qual
                confoptions=(--disable-cxx --disable-fortran --disable-dependency-tracking --enable-parallel)
                cppflags=(-I\${MPICH_FQ_DIR}/include)
                ldflags=(-L\${MPICH_FQ_DIR}/lib -lmpi)
                ;;
            mvapich2)
                ssi_die "mvapich2 is currently not supported: please contact artists@fnal.gov if you need this."
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, nofcpp and zlib are mutually exclusive"
                bqual=$qual
                confoptions=(--disable-cxx --disable-fortran --disable-dependency-tracking --enable-parallel)
                ;;
            nofcpp)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, nofcpp and zlib are mutually exclusive"
                bqual=$qual
                confoptions=(--disable-cxx --disable-fortran --disable-dependency-tracking)
                ;;
            zlib)
                (( seen_bqual++ )) && ssi_die "Qualifiers mpich, mvapich2, nofcpp and zlib are mutually exclusive"
                bqual=$qual
                confoptions=(--enable-cxx --enable-fortran --with-zlib=\${ZLIB_INC},\${ZLIB_LIB})
                ;;
            *)
                ssi_die "Unrecognized qualifier $qual"
        esac
    done
    if [[ -z "$cqual" ]]; then
        ssi_die "Base qualifier is mandatory"
    fi
    (( ${seen_bqual:-0} )) || ssi_die "zlib|nofcpp|mvapich2|mpich qualifier is required"
    if [[ -z "$extraqual" ]] && [[ "$bqual" != "zlib" ]]; then
      ssi_die "debug|prof is mandatory with $bqual"
    elif [[ -n "$zlib" ]] && [[ -n "$extraqual" ]]; then
      ssi_die "$extraqual not compatible with $zlib"
    fi
    fullqual=${cqual}:${bqual}${extraqual:+:${extraqual}}
    qualdir=`echo ${fullqual} | sed -e 's/:/-/g'`
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

set_compiler_flags()
{
    case ${cqual} in
        e6)
            cxxflags="-std=c++1y"
            ;;
        e[79]|e1[04])
            cxxflags="-std=c++14"
            ;;
        *)
            ssi_die "Unrecognized compiler qualifier $cqual"
    esac
    [[ -n "${extraqual}" ]] && 
    case x$extraqual in
        xdebug)
            prodflag=--enable-build-mode=debug
            ;;
        xprof)
            prodflag=--enable-build-mode=production
            # The presence of .dSym directories in production builds
            # causes a test to fail.
            [[ "$OS1" == "Darwin" ]] || gflag="-g"
            ;;
        x)
            ;;
        *)
            ssi_die "Unrecognized extra qualifier $extraqual"
    esac
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
if [[ "$extraqual" == "tar" ]]; then
  unset extraqual
  maketar=tar
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=hdf5
origpkgver=v1_10_0-patch1
pkgver=${origpkgver//-/_}b
ssibuildshims_version=v0_15_09
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir
get_ssibuildshims
source define_basics # From ssibuildshims.
set_qualifiers
get_platform_name
set_compiler_flags

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

ups active

set -x

cd "${sourcedir}" && rm -rf hdf5-${pkgdotver} || ssi_die "Unable to cd into ${sourcedir}"
tar xzf ${tardir}/${pkgtarname}* || ssi_die "Unable to untar source"

# Apply patch.
cd "${pkgtarname}" || ssi_die "Unable to cd into ${sourcedir}/${pkgtarname} for application of patch."
patch="hdf51.10-CVE2016.patch"
patch ${DEBUG_PATCHES:+--verbose} -p0 < "${patchdir}/${patch}" || ssi_die "application of patch ${patch} failed."
[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd "${blddir}" || ssi_die "Unable to cd into ${blddir}"

eval CC=gcc CXX=g++ \
     ${cppflags:+CPPFLAGS=\""${cppflags[@]}"\"} ${gflag:+CFLAGS=${gflag}} \
     ${cxxflags:+CXXFLAGS=${cxxflags}} ${ldflags:+LDFLAGS=\""${ldflags[@]}"\"} \
${sourcedir}/hdf5-${pkgdotver}/configure --prefix="${pkgdir}" "${confoptions[@]}" ${prodflag} || \
  ssi_die "configure failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

make -j ${ncore} || ssi_die "make failed"

(( ${HDF5_NO_CHECKS:-0} )) || make -j ${ncore} check || ssi_die "make check failed"

make install || ssi_die "make install failed"

(( ${HDF5_NO_CHECKS:-0} )) || make check-install || ssi_die "make check-install failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Update wrapper with correct prefix.
for f in ${pkgdir}/bin/h5{,p}cc; do
  [[ -w "${f}" ]] || continue
  perl -wapi\~ -e 's&^prefix=.*$&prefix="\${HDF5_FQ_DIR}"&' "${f}" || ssi_die "Unable to update wrapper scripts." && rm -f "${f}~"
done

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || \
  ssi_die "failed to declare ${package} ${pkgver} -q ${fullqual}"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "hdf5 is installed at ${HDF5_FQ_DIR}"


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
